@@ -27,6 +27,7 @@
 #include "th-title-selector.h"
 #include "th-utils.h"
 
+#include <gst/gst.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 
@@ -85,6 +86,8 @@ static void             appwin_action_show_log_cb   (GtkAction *action, ThAppWin
 
 static void             appwin_action_open_vob_folder_cb (GtkAction *action, ThAppWindow *appwin);
 
+static void             appwin_action_open_iso_file_cb (GtkAction *action, ThAppWindow *appwin);
+
 /* variables */
 
 static GtkWindowClass  *appwin_parent_class; /* NULL */
@@ -119,6 +122,13 @@ static GtkActionEntry appwin_action_entries[] =
     N_("Open a local folder containing .VOB and .IFO files from a video DVD"), 
     G_CALLBACK (appwin_action_open_vob_folder_cb)  
   },
+  { "OpenISOFile", 
+    GTK_STOCK_OPEN,  
+    N_("Open .iso DVD image _file"),  
+    NULL,
+    N_("Open a file containing a DVD in form of an .iso image"),
+    G_CALLBACK (appwin_action_open_iso_file_cb)
+  },
   { "Quit", 
     GTK_STOCK_QUIT,  
     N_("_Quit"),
@@ -154,6 +164,7 @@ static const gchar *appwin_ui_description =
 "  <menubar name='MainMenu'>"
 "    <menu action='FileMenu'>"
 "      <menuitem action='OpenVOBFolder'/>"
+"      <menuitem action='OpenISOFile'/>"
 "      <separator />"
 "      <menuitem action='Quit'/>"
 "    </menu>"
@@ -629,6 +640,45 @@ appwin_action_open_vob_folder_cb (GtkAction *action, ThAppWindow *appwin)
 	g_free (fn);
 }
 
+/***************************************************************************
+ *
+ *   appwin_action_open_iso_file_cb
+ *
+ ***************************************************************************/
+
+static void
+appwin_action_open_iso_file_cb (GtkAction *action, ThAppWindow *appwin)
+{
+	GtkFileFilter *filter;
+	GtkWidget     *fc;
+	gchar         *fn;
+
+	fc = gtk_file_chooser_dialog_new (NULL, /* "Load DVD .iso image ..." */
+	                                  GTK_WINDOW (appwin),
+	                                  GTK_FILE_CHOOSER_ACTION_OPEN, 
+	                                  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
+	                                  NULL);
+
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_add_mime_type (filter, "application/x-cd-image");
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (fc), filter);
+
+	if (gtk_dialog_run (GTK_DIALOG (fc)) != GTK_RESPONSE_ACCEPT)
+	{
+		gtk_widget_destroy (fc);
+		return;
+	}
+
+	fn = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fc));
+	gtk_widget_destroy (fc);
+
+	if (fn == NULL)
+		return;
+
+	th_title_selector_add_iso_file (TH_TITLE_SELECTOR (appwin->priv->ts), fn);
+	g_free (fn);
+}
+
 /***************************************************************************
  *
  *   appwin_about_url_func
@@ -642,6 +692,41 @@ appwin_about_url_func (GtkAboutDialog *about_dlg, const gchar *url, gpointer foo
 	(void) th_utils_invoke_browser_with_url (url);
 }
 
+/***************************************************************************
+ *
+ *   appwin_about_dialog_get_comments
+ *
+ ***************************************************************************/
+
+static gchar *
+appwin_about_dialog_get_comments (ThAppWindow *appwin)
+{
+	GstPlugin *core, *base, *good, *ugly;
+	gchar *versions, *ret;
+
+	core = gst_default_registry_find_plugin ("coreelements");
+	base = gst_default_registry_find_plugin ("audioconvert");
+	good = gst_default_registry_find_plugin ("videobox");
+	ugly = gst_default_registry_find_plugin ("dvdread");
+
+	versions = g_strdup_printf ("GStreamer core %s\n"
+	    "GStreamer base %s\nGStreamer good %s\nGStreamer ugly %s",
+	    gst_plugin_get_version (core), gst_plugin_get_version (base),
+	    gst_plugin_get_version (good), gst_plugin_get_version (ugly));
+
+	gst_object_unref (core);
+	gst_object_unref (base);
+	gst_object_unref (good);
+	gst_object_unref (ugly);
+
+	ret = g_strdup_printf ("%s\n\n%s",
+	    _("DVD video extraction and backup tool"), versions);
+
+	g_free (versions);
+
+	return ret;
+}
+
 /***************************************************************************
  *
  *   appwin_action_about_cb
@@ -658,6 +743,7 @@ appwin_action_about_cb (GtkAction *action, ThAppWindow *appwin)
 	
 	GtkAboutDialog *dlg;
 	GString        *fn;
+	gchar          *comments;
 	gchar          *license = NULL;
 	
 	dlg = (GtkAboutDialog*) gtk_about_dialog_new ();
@@ -723,6 +809,10 @@ appwin_action_about_cb (GtkAction *action, ThAppWindow *appwin)
 		}
 	}
 
+	comments = appwin_about_dialog_get_comments (appwin);
+	gtk_about_dialog_set_comments (dlg, comments);
+	g_free (comments);
+
 	(void) gtk_dialog_run (GTK_DIALOG (dlg));
 	
 	gtk_widget_destroy (GTK_WIDGET (dlg));
