@@ -22,6 +22,8 @@
 #include "th-disc-drive.h"
 #include "th-utils.h"
 
+#include <gst/gstregistry.h>
+
 #include <glib/gi18n.h>
 
 #include "_stdint.h"
@@ -44,7 +46,7 @@
  *  (= be shown in the list) in seconds */
 #define MINIMUM_JOB_DURATION   15
 
-/* I think 8 audio stremas is the maximum */
+/* 8 audio streams is the maximum */
 #define MAX_AUDIO_STREAMS 8
 
 /* how often we poll for a media 
@@ -131,7 +133,9 @@ struct _ThTitleInfo
 	guint num_audio_streams; /* number of (interesting) audio streams */
 	struct 
 	{
-		guint         aid;
+		guint         aid;      /* this is the _logical_ audio stream number */
+		guint         phys_aid; /* this is the physical stream number on which
+		                         * the mpeg demuxer will base the pad name   */
 		audio_attr_t  attr;
 	} 
 	audio_streams[MAX_AUDIO_STREAMS];
@@ -553,7 +557,7 @@ dd_dump_title_info (ThTitleInfo *info)
 
 	for (j = 0;  j < info->num_audio_streams;  ++j)
 	{
-		th_log ("      [Audio %02u] [%s] %s, %s\n", info->audio_streams[j].aid,
+		th_log ("      [Audio %02u] [%s] %s, %s\n", info->audio_streams[j].phys_aid,
 		         dd_audio_stream_get_language_str_short (&info->audio_streams[j].attr),
 		         dd_audio_stream_get_format_str (&info->audio_streams[j].attr),
 		         dd_audio_stream_get_channel_str (&info->audio_streams[j].attr));
@@ -576,7 +580,7 @@ dd_job_add_audio_streams (ThJob *job, ThTitleInfo *info)
 	{
 		audio_attr_t *attr;
 		const gchar  *lang_str, *chan_str, *fmt_str;
-		gchar        *desc;
+		gchar        *desc, *pad_name;
 		
 		attr = &info->audio_streams[i].attr;
 		
@@ -585,8 +589,10 @@ dd_job_add_audio_streams (ThJob *job, ThTitleInfo *info)
 		lang_str = dd_audio_stream_get_language_str_short (attr);
 		
 		desc = g_strdup_printf ("(%s, %s)", fmt_str, chan_str);
-
-		th_job_add_audio_stream (job, info->audio_streams[i].aid, lang_str, desc);
+		pad_name = g_strdup_printf ("audio_%02u", info->audio_streams[i].phys_aid);
+		th_job_add_audio_stream (job, info->audio_streams[i].aid,
+		                         pad_name, lang_str, desc);
+		g_free (pad_name);
 		g_free (desc);
 	}
 }
@@ -691,27 +697,34 @@ dd_ioc_read_cb (GIOChannel *ioc, GIOCondition cond, ThDiscDrive *drive)
 		GIOStatus  ret;
 		GError    *err = NULL;
 		gchar      buf[8192];
-		gsize      num_read = 0;
-		
-		ret = g_io_channel_read_chars (drive->priv->ioc, buf, sizeof(buf), &num_read, &err);
-		
-		if (err)
+		gsize      num_read;
+
+		do
 		{
-			th_log ("error in dd_ioc_read_cb(): %s\n", err->message);
-			g_error_free (err);
-		}
+			num_read = 0;
+
+			ret = g_io_channel_read_chars (drive->priv->ioc, buf,
+			                               sizeof (buf), &num_read, &err);
 		
-		if (ret != G_IO_STATUS_NORMAL)
-			cond |= G_IO_ERR;
+			if (err)
+			{
+				th_log ("error in dd_ioc_read_cb(): %s\n", err->message);
+				g_error_free (err);
+			}
+		
+			if (ret != G_IO_STATUS_NORMAL)
+				cond |= G_IO_ERR;
 			 
-		if (num_read < 0)
-			cond |= G_IO_HUP;
+			if (num_read <= 0)
+				cond |= G_IO_HUP;
 		
-		if (num_read > 0)
-		{
-			th_log ("pipe read %u bytes\n", (guint) num_read);
-			g_string_append_len (drive->priv->ioc_buf, buf, num_read);
+			if (num_read > 0)
+			{
+				th_log ("pipe read %u bytes\n", (guint) num_read);
+				g_string_append_len (drive->priv->ioc_buf, buf, num_read);
+			}
 		}
+		while (ret == G_IO_STATUS_NORMAL && num_read == sizeof (buf));
 	}
 	
 	if ((cond & (G_IO_ERR | G_IO_HUP | G_IO_NVAL)))
@@ -844,32 +857,25 @@ dd_audio_format_supported (guint audio_format)
 	
 	if (checked == FALSE)
 	{
-		/* Linear PCM only works with gst-plugins CVS after Jan 25, 2005,
-		 * DTS only works with gst-plugins CVS after Feb 10, 2005 */
-		if (th_utils_check_plugin_version ("mpegstream", "dvddemux", 0, 8, 7, 1))
+		if (gst_default_registry_check_feature_version ("dvdlpcmdec", 0, 10, 0))
 		{
-			th_log ("dvddemux probably supports Linear PCM");
-			th_log ("dvddemux probably supports DTS");
+			th_log ("dvdlpcmdec plugin available");
+			lpcm_ok = TRUE;
+		} else th_log ("dvdlpcmdec plugin not available");
 
-			if (th_utils_check_plugin_version ("dvdlpcmdec", "dvdlpcmdec", 0, 8, 7, 1))
-			{
-				th_log ("dvdlpcmdec plugin available");
-				lpcm_ok = TRUE;
-			} else th_log ("dvdlpcmdec plugin not available");
+		if (gst_default_registry_check_feature_version ("dtsdec", 0, 10, 0))
+		{
+			th_log ("dtsdec plugin available");
+			dts_ok = TRUE;
+		} else th_log ("dtsdec plugin not available");
 
-			if (th_utils_check_plugin_version ("dtsdec", "dtsdec", 0, 8, 7, 1))
-			{
-				th_log ("dtsdec plugin available");
-				dts_ok = TRUE;
-			} else th_log ("dtsdec plugin not available");
+		/* ffdec_mp3 isn't tested well enough and mad is hardcoded in th-job.c */
+		if (gst_default_registry_check_feature_version ("mad", 0, 10, 0))
+		{
+			th_log ("mad plugin available");
+			mp3_ok = TRUE;
+		} else th_log ("mad plugin not available");
 
-			/* ffdec_mp3 isn't tested well enough and mad is hardcoded in th-job.c */
-			if (th_utils_check_plugin_version ("mad", "mad", 0, 8, 7, 1))
-			{
-				th_log ("mad plugin available");
-				mp3_ok = TRUE;
-			} else th_log ("mad plugin not available");
-		}
 		checked = TRUE;
 	}
 
@@ -911,19 +917,21 @@ disc_drive_get_title_set_audio_info (ThDiscDrive  *drive,
 
 	th_log ("\n");
 
-	/* vts_file->vtsi_mat->nr_of_vts_audio_streams */
-	for (i = 0;  i < 8;  ++i)
+	for (i = 0;  i < vts_file->vtsi_mat->nr_of_vts_audio_streams;  ++i)
 	{
 		audio_attr_t  attr;
+		guint phys_streamid;
 
-		/* not a valid audio stream? */ 
+		/* audio stream not present? */ 
 		if ((pgc->audio_control[i] & 0x8000) == 0)
 			continue;
 
+		phys_streamid = (pgc->audio_control[i] & 0x700) >> 8;
+
 		attr = vts_file->vtsi_mat->vts_audio_attr[i];
-	
-		th_log ("%3u  audio_%02u  %s  %s, %-8s  (type = %u, ext = %u)\n", 
-		         tinfo->num + 1, i, 
+
+		th_log ("%3u  audio_%02u (%02u)  %s  %s, %-8s  (type = %u, ext = %u)\n", 
+		         tinfo->num + 1, phys_streamid, i, 
 			       dd_audio_stream_get_language_str_short (&attr),
 			       dd_audio_stream_get_format_str (&attr),
 			       dd_audio_stream_get_channel_str (&attr),
@@ -947,6 +955,7 @@ disc_drive_get_title_set_audio_info (ThDiscDrive  *drive,
 					if (dd_audio_stream_compare (&tinfo->audio_streams[j].attr, &attr) > 0)
 					{
 						tinfo->audio_streams[j].aid  = i;
+						tinfo->audio_streams[j].phys_aid  = phys_streamid;
 						tinfo->audio_streams[j].attr = attr;
 					}
 					break; /* => this stream will not be added below */
@@ -957,6 +966,7 @@ disc_drive_get_title_set_audio_info (ThDiscDrive  *drive,
 			if (j == tinfo->num_audio_streams)
 			{
 				tinfo->audio_streams[tinfo->num_audio_streams].aid  = i;
+				tinfo->audio_streams[tinfo->num_audio_streams].phys_aid  = phys_streamid;
 				tinfo->audio_streams[tinfo->num_audio_streams].attr = attr;
 				
 				++tinfo->num_audio_streams;
