@@ -195,7 +195,8 @@ job_config_dialog_run_cropping_dialog (ThJobConfigDialog *jcd, GtkButton *b)
 	GtkWidget *appwin;
 
 	appwin = gtk_widget_get_toplevel (GTK_WIDGET (jcd));
-	(void) th_job_cropping_dialog_run_for_job (jcd->priv->job, GTK_WINDOW (appwin));
+
+	th_job_cropping_dialog_run_for_job (jcd->priv->job, GTK_WINDOW (appwin));
 	
 	job_config_dialog_update_crop_labels (jcd);
 	job_config_dialog_job_update_picsizes (jcd);
