@@ -17,6 +17,7 @@
 
 #include "th-job-progress-dialog.h"
 #include "th-job.h"
+#include "th-power-manager-agent.h"
 #include "th-utils.h"
 
 #include <gtk/gtk.h>
@@ -68,6 +69,8 @@ struct _ThJobProgressDialogPrivate
 
 	/* prefs */
 	gboolean      delete_partial_file_on_cancel;
+
+	ThPowerManagerAgent *power_manager_agent;
 };
 
 static void             job_progress_dialog_class_init    (ThJobProgressDialogClass *klass);
@@ -239,6 +242,8 @@ job_progress_dialog_instance_init (ThJobProgressDialog *jpd)
 	jpd->priv->job_timer = g_timer_new ();
 	jpd->priv->fps_timer = g_timer_new ();
 
+	jpd->priv->power_manager_agent = th_power_manager_agent_new ();
+
 	jpd->priv->response_id = g_signal_connect (jpd, "response", 
 	                            G_CALLBACK (job_progress_dialog_response), 
 	                            NULL);
@@ -267,6 +272,7 @@ job_progress_dialog_finalize (GObject *object)
 	th_log ("ThJobProgressDialog: finalize\n");
 
 	g_object_unref (jpd->priv->empty_pixbuf);
+	g_object_unref (jpd->priv->power_manager_agent);
 
 	g_timer_destroy (jpd->priv->job_timer);
 	g_timer_destroy (jpd->priv->fps_timer);
@@ -435,6 +441,9 @@ job_progress_dialog_response (ThJobProgressDialog *jpd, gint ret, gpointer foo)
 		return;
 	}
 
+	if (ret == TH_RESPONSE_DESTROYED)
+		goto cancel_job;
+
 	/* will be 'Cancel' or 'Close' */
 
 	if (GTK_WIDGET_VISIBLE (jpd->priv->cancel_button) 
@@ -460,6 +469,8 @@ job_progress_dialog_response (ThJobProgressDialog *jpd, gint ret, gpointer foo)
 			return;
 	}
 
+cancel_job:
+
 	th_job_cancel (jpd->priv->job, jpd->priv->delete_partial_file_on_cancel);
 }
 
@@ -978,6 +989,7 @@ job_progress_dialog_run_job (ThJobProgressDialog *jpd, ThJob *job, gboolean *p_c
 		GtkWidget *dlg, *appwin;
 		
 		appwin = gtk_widget_get_toplevel (GTK_WIDGET (jpd));
+		g_assert (GTK_IS_WINDOW (appwin));
 		dlg = gtk_message_dialog_new (GTK_WINDOW (appwin),
 		                              GTK_DIALOG_MODAL,
 		                              GTK_MESSAGE_ERROR,
@@ -1024,22 +1036,6 @@ job_progress_dialog_set_job_progress_bar (ThJobProgressDialog *jpd, guint n, gui
 	}
 }
 
-/***************************************************************************
- *
- *   jpd_free_snapshot_frames
- *
- ***************************************************************************/
-
-static gboolean
-jpd_free_snapshot_frames (GList *jobs)
-{
-	GList *l;
-	for (l = jobs; l; l = l->next)
-		th_job_set_snapshot_frames (TH_JOB (l->data), NULL);
-
-	return TRUE; /* call again (to keep the id valid) */
-}
-
 /***************************************************************************
  *
  *   th_job_progress_dialog_run_jobs
@@ -1051,12 +1047,15 @@ th_job_progress_dialog_run_jobs (ThJobProgressDialog *jpd, GList *jobs)
 {
 	gboolean  got_cancelled = FALSE;
 	guint     num_jobs, n = 0;
-	gulong    timeout = 0;
 	GList    *l;
 	
 	g_return_val_if_fail (TH_IS_JOB_PROGRESS_DIALOG (jpd), FALSE);
 	g_return_val_if_fail (jobs != NULL, FALSE);
 
+	g_object_set (jpd->priv->power_manager_agent,
+	              "inhibit-power-manager", TRUE,
+	              NULL);
+
 	gtk_widget_hide (jpd->priv->close_button);
 	gtk_widget_show (jpd->priv->pause_button);
 	gtk_widget_show (jpd->priv->cancel_button);
@@ -1073,8 +1072,6 @@ th_job_progress_dialog_run_jobs (ThJobProgressDialog *jpd, GList *jobs)
 		jpd->priv->weight_total += w;
 	}
 
-	timeout = g_timeout_add (60 * 1000, (GSourceFunc) jpd_free_snapshot_frames, jobs);
-
 	for (l = jobs;  l;  l = l->next)
 	{
 		ThJob *job = TH_JOB (l->data);
@@ -1083,7 +1080,7 @@ th_job_progress_dialog_run_jobs (ThJobProgressDialog *jpd, GList *jobs)
 
 		if (!job_progress_dialog_run_job (jpd, job, &got_cancelled)  &&  got_cancelled)
 		{
-			g_source_remove (timeout);
+			/* the dialog object has been destroyed now */
 			return FALSE;
 		}
 		
@@ -1107,7 +1104,9 @@ th_job_progress_dialog_run_jobs (ThJobProgressDialog *jpd, GList *jobs)
 	
 	gtk_widget_hide (GTK_WIDGET (jpd));
 
-	g_source_remove (timeout);
+	g_object_set (jpd->priv->power_manager_agent,
+	              "inhibit-power-manager", FALSE,
+	              NULL);
 
 	return TRUE;
 }
