@@ -22,6 +22,7 @@
 #include "th-device-pool.h"
 #include "th-disc-drive-pool.h"
 #include "th-job.h"
+#include "th-title-preview.h"
 #include "th-title-selector.h"
 #include "th-utils.h"
 
@@ -30,9 +31,6 @@
 #include <glib/gi18n.h>
 #include <string.h>
 
-#define PREVIEW_WIDTH  384
-#define PREVIEW_HEIGHT 288
-
 typedef enum
 {
 	MSG_DETECTING_DRIVES,
@@ -72,15 +70,8 @@ struct _ThTitleSelectorPrivate
 	GtkWidget        *ok_button;
 	GtkWidget        *cancel_button;
 
-	GtkWidget        *preview_win;
-	GtkWidget        *preview_img;
-	GstElement       *preview_pipeline;
-	GMainLoop        *preview_loop;
-	gulong            preview_id;
-	guint             preview_title_num;
-	const gchar      *preview_title_tag;
-	GTimer           *preview_timer;
-	
+	GtkWidget        *preview;
+
 	TitleListMsg      cur_msg;
 };
 
@@ -236,8 +227,6 @@ title_selector_finalize (GObject *object)
 
 	th_log ("ThTitleSelector: finalize\n");
 	
-	g_assert (ts->priv->preview_loop == NULL);
-
 	g_object_unref (ts->priv->dev_pool);
 
 	g_object_unref (ts->priv->drive_list);
@@ -303,8 +292,10 @@ title_selector_response_cb (ThTitleSelector *ts, gint r, GtkWidget *w)
 {
 	g_return_if_fail (TH_IS_TITLE_SELECTOR (ts));
 	
-	if (ts->priv->preview_loop)
-		g_main_loop_quit (ts->priv->preview_loop);
+	if (ts->priv->preview) {
+		gtk_widget_destroy (ts->priv->preview);
+		ts->priv->preview = NULL;
+	}
 }
 
 /***************************************************************************
@@ -600,125 +591,6 @@ title_selector_titles_check_cb (ThTitleSelector *ts)
 	return TRUE; /* call again */
 }
 
-/***************************************************************************
- *
- *   ts_preview_error_cb
- *
- ***************************************************************************/
-
-static void
-ts_preview_error_cb (ThTitleSelector *ts, GstElement *el, GError *gerr, const gchar *dbg, GstElement *bin)
-{
-	/* just stop the pipeline from iterating, 
-	 *  but keep the window where it is */
-	g_source_remove (ts->priv->preview_id);
-	ts->priv->preview_id = 0;
-	
-	gtk_window_set_title (GTK_WINDOW (ts->priv->preview_win), gerr->message);
-	
-	g_warning ("Preview pipeline error: %s\n", gerr->message);
-	
-	g_main_loop_quit (ts->priv->preview_loop);
-}
-
-/***************************************************************************
- *
- *   ts_preview_eos_cb
- *
- ***************************************************************************/
-
-static void
-ts_preview_eos_cb (ThTitleSelector *ts, GstElement *el)
-{
-	g_main_loop_quit (ts->priv->preview_loop);
-}
-
-/***************************************************************************
- *
- *   ts_preview_delete_cb
- *
- ***************************************************************************/
-
-static gboolean
-ts_preview_delete_cb (ThTitleSelector *ts, GdkEvent *ev, GtkWidget *win)
-{
-	g_main_loop_quit (ts->priv->preview_loop);
-	return TRUE; /* do not destroy, we'll do that ourselves */
-}
-
-/***************************************************************************
- *
- *   ts_preview_idle_iterate_cb
- *
- ***************************************************************************/
-
-static gboolean
-ts_preview_idle_iterate_cb (ThTitleSelector *ts)
-{
-	if (!gst_bin_iterate (GST_BIN (ts->priv->preview_pipeline)))
-		; //g_usleep (500); /* eek */
-	return TRUE; /* call again */
-}
-
-/***************************************************************************
- *
- *   ts_preview_handoff_cb
- *
- ***************************************************************************/
-
-static void
-ts_preview_handoff_cb (ThTitleSelector *ts, GstBuffer *buf, GstPad *pad, GstElement *fakesink)
-{
-	static guint  secs_last; /* 0 */
-	GdkPixbuf    *pixbuf;
-	guint         secs;
-
-	secs = GST_BUFFER_TIMESTAMP (buf) / GST_SECOND;
-	
-	if (ts->priv->preview_timer == NULL)
-	{
-		ts->priv->preview_timer = g_timer_new ();
-	}
-	else
-	{
-		/* skip frames if we're behind */
-		if (g_timer_elapsed (ts->priv->preview_timer, NULL) 
-		    > (GST_BUFFER_TIMESTAMP (buf) / ((gdouble)GST_SECOND)))
-			return;
-	}
-
-	pixbuf = th_pixbuf_from_yuv_i420_buffer (buf, PREVIEW_WIDTH, PREVIEW_HEIGHT);
-	if (pixbuf)
-	{
-		gtk_image_set_from_pixbuf (GTK_IMAGE (ts->priv->preview_img), pixbuf);
-		g_object_unref (pixbuf);
-	}
-
-	if (secs != secs_last || secs_last == 0)
-	{
-		gchar title[256];
-		guint h, m, s;
-		
-		h = secs / 3600;
-		m = (secs % 3600) / 60;
-		s = secs % 60;
-
-		g_snprintf (title, sizeof (title), 
-		            _("[%u:%02u:%02u] Title %u"), 
-		            h, m, s, ts->priv->preview_title_num + 1);
-
-		if (ts->priv->preview_title_tag == NULL)
-			ts->priv->preview_title_tag = "";
-
-		g_strlcat (title, ":", sizeof (title));
-		g_strlcat (title, ts->priv->preview_title_tag, sizeof (title));
-		
-		gtk_window_set_title (GTK_WINDOW (ts->priv->preview_win), title);
-
-		secs_last = secs;
-	}
-}
-
 /***************************************************************************
  *
  *   title_selector_preview_title
@@ -728,11 +600,9 @@ ts_preview_handoff_cb (ThTitleSelector *ts, GstBuffer *buf, GstPad *pad, GstElem
 static void
 title_selector_preview_title (ThTitleSelector *ts, ThJob *job)
 {
-	GstElement  *fakesink, *src;
-	GtkWidget   *appwin;
-	GError      *err = NULL;
-	gchar       *desc, *dev, *title_tag;
-	guint        title_num;
+	GError *err = NULL;
+	gchar  *device, *title_tag;
+	guint   title_num;
 
 	g_return_if_fail (TH_IS_JOB (job));
 
@@ -741,27 +611,21 @@ title_selector_preview_title (ThTitleSelector *ts, ThJob *job)
 	              "title-tag", &title_tag, 
 	              NULL);
 
-	g_object_get (ts->priv->active_drive, "device", &dev, NULL);
+	g_object_get (ts->priv->active_drive, "device", &device, NULL);
 
-	/* hmm, maybe we should just use xvimagesink instead?
-	 *  gotta find out how to get the XID and set the 
-	 *  window title then. Both xvimagesink and ximagesink
-	 *  seem a bit broken though at the moment */
-	desc = g_strdup_printf ("dvdreadsrc title=%u name=src"
-	                        " ! dvddemux"
-	                        " ! mpeg2dec"
-	                        " ! ffmpegcolorspace"
-	                        " ! videoscale"
-	                        " ! video/x-raw-yuv,width=%u,height=%u"
-	                        " ! fakesink name=fakesink sync=TRUE signal-handoffs=TRUE",
-	                       title_num, PREVIEW_WIDTH, PREVIEW_HEIGHT);
-	                        
-	th_log ("Launching pipeline:\n  %s\n\n", desc);
 
-	ts->priv->preview_pipeline = gst_parse_launch (desc, &err);
-	
-	if (err || ts->priv->preview_pipeline == NULL)
-	{
+	if (ts->priv->preview) {
+		gtk_widget_destroy (ts->priv->preview);
+		ts->priv->preview = NULL;
+	}
+
+	ts->priv->preview =
+	    th_title_preview_new (device, title_tag, title_num + 1, &err);
+
+	g_free (title_tag);
+	g_free (device);
+
+	if (err != NULL || ts->priv->preview == NULL) {
 		GtkWidget *dlg, *appwin;
 
 		appwin = gtk_widget_get_toplevel (GTK_WIDGET (ts));
@@ -770,101 +634,29 @@ title_selector_preview_title (ThTitleSelector *ts, ThJob *job)
 		                              GTK_DIALOG_MODAL,
 		                              GTK_MESSAGE_ERROR,
 		                              GTK_BUTTONS_OK,
-		                              _("Could not run preview pipeline\n\n%s\n\n%s"),
-		                              desc, (err) ? err->message : _("unknown error"));
+		                              _("Can't preview DVD title %d"),
+		                              title_num + 1);
+
+		gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dlg),
+		    "%s", (err) ? err->message : _("unknown error"));
+
 		(void) gtk_dialog_run (GTK_DIALOG (dlg));
 		gtk_widget_destroy (dlg);
 		if (err)                             
 			g_error_free (err);
-		if (ts->priv->preview_pipeline)
-		{
-			gst_object_unref (GST_OBJECT(ts->priv->preview_pipeline));
-			ts->priv->preview_pipeline = NULL;
-		}
-		g_free (desc);
-		g_free (dev);
-		g_free (title_tag);
 		return;
-	} 
-	
-	fakesink = gst_bin_get_by_name (GST_BIN (ts->priv->preview_pipeline), "fakesink");
-	g_return_if_fail (GST_IS_ELEMENT (fakesink));
-
-	src = gst_bin_get_by_name (GST_BIN (ts->priv->preview_pipeline), "src");
-	g_return_if_fail (src != NULL);
-
-	/* we use g_object_set() here instead of specifying this in
-	 * the pipeline above to avoid escaping issues when we are 
-	 * using a local directory path instead of a DVD device */
-	g_object_set (src, "device", dev, NULL);
-	
-	ts->priv->preview_title_num = title_num;
-	ts->priv->preview_title_tag = title_tag;
-
-	ts->priv->preview_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	ts->priv->preview_img = gtk_image_new ();
-	
-	gtk_window_set_default_size (GTK_WINDOW (ts->priv->preview_win), 
-	                             PREVIEW_WIDTH, PREVIEW_HEIGHT);
-	
-	gtk_window_set_type_hint (GTK_WINDOW (ts->priv->preview_win), GDK_WINDOW_TYPE_HINT_UTILITY);
-
-	appwin = gtk_widget_get_toplevel (GTK_WIDGET (ts));
-	gtk_window_set_transient_for (GTK_WINDOW (ts->priv->preview_win), GTK_WINDOW (appwin));
-	
-	gtk_container_add (GTK_CONTAINER (ts->priv->preview_win), ts->priv->preview_img);
-	gtk_widget_show_all (ts->priv->preview_win);
-	
-	g_signal_connect_swapped (ts->priv->preview_win, "delete-event", 
-	                          G_CALLBACK (ts_preview_delete_cb),
-	                          ts);
-
-	g_signal_connect_swapped (ts->priv->preview_pipeline, "eos", 
-	                          G_CALLBACK (ts_preview_eos_cb), 
-	                          ts);
-
-	g_signal_connect_swapped (ts->priv->preview_pipeline, "error", 
-	                          G_CALLBACK (ts_preview_error_cb), 
-	                          ts);
-
-	g_signal_connect_swapped (fakesink, "handoff", 
-	                          G_CALLBACK (ts_preview_handoff_cb), 
-	                          ts);
-	
-	ts->priv->preview_id = g_idle_add_full (G_PRIORITY_LOW,
-	                                        (GSourceFunc) ts_preview_idle_iterate_cb, 
-	                                        ts, NULL);
-	
-	ts->priv->preview_loop = g_main_loop_new (NULL, FALSE);
-
-	gst_element_set_state (GST_ELEMENT (ts->priv->preview_pipeline), GST_STATE_PLAYING);
-	
-	g_main_loop_run (ts->priv->preview_loop);
-	
-	gst_element_set_state (GST_ELEMENT (ts->priv->preview_pipeline), GST_STATE_NULL);
-	
-	gst_object_unref (GST_OBJECT (ts->priv->preview_pipeline));
-	ts->priv->preview_pipeline = NULL;
-	
-	g_main_loop_unref (ts->priv->preview_loop);
-	ts->priv->preview_loop = NULL;
-
-	if (ts->priv->preview_id > 0)
-	{
-		g_source_remove (ts->priv->preview_id);
-		ts->priv->preview_id = 0;
 	}
-	
-	gtk_widget_destroy (ts->priv->preview_win);
-	ts->priv->preview_win = NULL;
 
-	if (ts->priv->preview_timer)
-		g_timer_destroy (ts->priv->preview_timer);
-	ts->priv->preview_timer = NULL;
-	
-	g_free (desc);
-	g_free (dev);
-	g_free (title_tag);
+	/* make sure ts->priv->preview is automatically set to NULL if it
+	 * gets destroyed other than by us (e.g. via close button) */
+	{
+		GtkWidget **widget_addr = &ts->priv->preview;
+
+		g_object_add_weak_pointer (G_OBJECT (ts->priv->preview),
+		                           (gpointer *) widget_addr);
+	}
+
+	gtk_widget_show (ts->priv->preview);
 }
 
 /***************************************************************************
@@ -931,7 +723,7 @@ title_selector_popup_context_menu (ThTitleSelector *ts, GdkEventButton *event)
 	sel = gtk_tree_view_get_selection (ts->priv->titles_view);
 	
 	if (!gtk_tree_selection_get_selected (sel, NULL, &iter)
-	  || ts->priv->preview_loop != NULL)
+	  || ts->priv->preview != NULL)
 	{
 		gtk_widget_set_sensitive (menuitem, FALSE);
 	}
@@ -1442,19 +1234,21 @@ title_selector_drive_data_func (GtkCellLayout   *cl,
 static void
 title_selector_on_drive_changed (ThTitleSelector *ts, GtkComboBox *combobox)
 {
+	ThDiscDrive **drive_addr;
 	GtkTreeIter  iter;
 	gint         row;
 	
 	g_return_if_fail (TH_IS_TITLE_SELECTOR (ts));
 	g_return_if_fail (GTK_IS_COMBO_BOX (combobox));
-	
+
+	drive_addr = &ts->priv->active_drive;
+
 	g_object_get (combobox, "active", &row, NULL);
 	
 	if (ts->priv->active_drive)
 	{
 		g_object_remove_weak_pointer (G_OBJECT (ts->priv->active_drive),
-		                              (gpointer *) &ts->priv->active_drive);
-		
+		                              (gpointer *) drive_addr);
 		ts->priv->active_drive = NULL;
 	}
 	
@@ -1476,7 +1270,7 @@ title_selector_on_drive_changed (ThTitleSelector *ts, GtkComboBox *combobox)
 	g_object_unref (ts->priv->active_drive);
 
 	g_object_add_weak_pointer (G_OBJECT (ts->priv->active_drive),
-	                           (gpointer *) &ts->priv->active_drive);
+	                           (gpointer *) drive_addr);
 
 	th_log ("active drive now: %p\n", ts->priv->active_drive);
 	ts->priv->cur_msg = MSG_RETRIEVING_DETAILS;
@@ -1780,3 +1574,40 @@ th_title_selector_add_vob_folder (ThTitleSelector *ts, const gchar *dir)
 	return TRUE;
 }
 
+/***************************************************************************
+ *
+ *   th_title_selector_add_iso_file
+ *
+ ***************************************************************************/
+
+gboolean
+th_title_selector_add_iso_file (ThTitleSelector *ts, const gchar *iso_fn)
+{
+	ThDiscDrive *drive;
+	gchar *base_name;
+
+	g_return_val_if_fail (TH_IS_TITLE_SELECTOR (ts), FALSE);
+	g_return_val_if_fail (iso_fn != NULL, FALSE);
+	g_return_val_if_fail (g_path_is_absolute (iso_fn), FALSE);
+
+	base_name = g_path_get_basename (iso_fn);
+
+	/* FIXME: find nice string and make translatable */
+	drive = th_disc_drive_new (iso_fn, iso_fn, "ISO image", "");
+
+	g_return_val_if_fail (drive != NULL, FALSE);
+
+	g_signal_emit_by_name (ts->priv->disc_pool, "drive-added", drive);
+
+	g_object_set (drive, "disc-title", base_name, NULL);
+
+	g_signal_emit_by_name (ts->priv->disc_pool, "disc-changed", drive);
+
+	/* the title selector list store holds a reference now */
+	g_object_unref (drive);
+
+	g_free (base_name);
+
+	return TRUE;
+}
+
