@@ -1140,144 +1140,21 @@ th_utils_get_plugin_rank (const gchar *plugin_name,  /* e.g. 'gstelements' */
                           const gchar *feature_name) /* e.g. 'filesrc'     */
 {
 	GstPluginFeature *plugin_feature;
-	GstPlugin        *plugin;
+	GstRegistry      *registry;
+	gint              rank = -1;
 
 	g_return_val_if_fail (plugin_name != NULL, -1);
 	g_return_val_if_fail (feature_name != NULL, -1);
 
-	plugin = gst_registry_pool_find_plugin (plugin_name);
-	if (plugin == NULL)
-		return -1;
-
-	plugin_feature = gst_plugin_find_feature (plugin, feature_name, 0);
-	if (plugin_feature == NULL)
-		return -1;
-	
-	return (gint) gst_plugin_feature_get_rank (plugin_feature);
-}
-
-/******************************************************************************
- *
- *   th_utils_get_plugin_version
- *
- *   Queries the version of a plugin. Returns TRUE if the
- *    plugin could be found and the version could be
- *    retrieved, otherwise FALSE.
- *
- ******************************************************************************/
-
-gboolean
-th_utils_get_plugin_version (const gchar *plugin_name,  /* e.g. 'gstelements' */
-                             const gchar *feature_name, /* e.g. 'filesrc'     */
-                             guint       *p_major,
-                             guint       *p_minor,
-                             guint       *p_micro,
-                             guint       *p_nano)
-{
-	const gchar *ver_str; 
-	GstPlugin   *plugin;
-	guint        major, minor, micro, nano;
-	
-	g_return_val_if_fail (plugin_name != NULL, FALSE);
-	g_return_val_if_fail (feature_name != NULL, FALSE);
-
-	plugin = gst_registry_pool_find_plugin (plugin_name);
-	if (plugin == NULL)
-		return FALSE;
-
-	if (!gst_plugin_find_feature (plugin, feature_name, 0))
-		return FALSE;
-
-	ver_str = gst_plugin_get_version (plugin);
-	if (ver_str == NULL)
-		return FALSE;
-	
-	if (sscanf (ver_str, "%u.%u.%u.%u", &major, &minor, &micro, &nano) != 4)
+	registry = gst_registry_get_default ();
+	plugin_feature = gst_registry_lookup_feature (registry, feature_name);
+	if (plugin_feature)
 	{
-		nano = 0;
-		if (sscanf (ver_str, "%u.%u.%u", &major, &minor, &micro) != 3)
-			return FALSE;
+		rank = (gint) gst_plugin_feature_get_rank (plugin_feature);
+		gst_object_unref (plugin_feature);
 	}
 	
-	if (p_major)
-		*p_major = major;
-	if (p_minor)
-		*p_minor = minor;
-	if (p_micro)
-		*p_micro = micro;
-	if (p_nano)
-		*p_nano = nano;
-
-	return TRUE;
-}
-
-/******************************************************************************
- *
- *   th_utils_check_version
- *
- ******************************************************************************/
-
-gboolean
-th_utils_check_version (guint detected_major,
-                        guint detected_minor,
-                        guint detected_micro,
-                        guint detected_nano,
-                        guint required_major,
-                        guint required_minor,
-                        guint required_micro,
-                        guint required_nano)
-{
-	if (detected_major > required_major)
-		return TRUE;
-
-	if (detected_major < required_major)
-		return FALSE;
-	
-	if (detected_minor > required_minor)
-		return TRUE;
-
-	if (detected_minor < required_minor)
-		return FALSE;
-	
-	if (detected_micro > required_micro)
-		return TRUE;
-	
-	if (detected_micro < required_micro)
-		return FALSE;
-	
-	return (detected_nano >= required_nano);
-}
-
-/******************************************************************************
- *
- *   th_utils_check_plugin_version
- *
- ******************************************************************************/
-
-gboolean
-th_utils_check_plugin_version (const gchar *plugin_name,  /* e.g. 'gstelements' */
-                               const gchar *feature_name, /* e.g. 'filesrc'     */
-                               guint        required_major,
-                               guint        required_minor,
-                               guint        required_micro,
-                               guint        required_nano)
-{
-	guint major = 0, minor = 0, micro = 0, nano = 0;
-
-	if (!th_utils_get_plugin_version (plugin_name, feature_name, 
-	                                  &major, &minor, &micro, &nano))
-	{
-		return FALSE;
-	}
-
-	if (!th_utils_check_version (major, minor, micro, nano, 
-	                             required_major, required_minor, 
-	                             required_micro, required_nano))
-	{
-		return FALSE;
-	}
-	
-	return TRUE;
+	return rank;
 }
 
 /******************************************************************************
@@ -1628,3 +1505,30 @@ th_get_user_config_fn (const gchar *filename)
 {
 	return get_user_foo_fn (filename, g_get_user_config_dir);
 }
+
+/***************************************************************************
+ *
+ *   th_bin_set_child_properties
+ *
+ ***************************************************************************/
+
+void
+th_bin_set_child_properties (GstBin *bin, const gchar *child_name,
+                             const gchar *prop1_name, ...)
+{
+	GstElement *child;
+	va_list     args;
+
+	g_return_if_fail (GST_IS_BIN (bin));
+	g_return_if_fail (child_name != NULL);
+	g_return_if_fail (prop1_name != NULL);
+
+	child = gst_bin_get_by_name (bin, child_name);
+	g_return_if_fail (child != NULL);
+
+	va_start (args, prop1_name);
+	g_object_set_valist (G_OBJECT (child), prop1_name, args);
+	va_end (args);
+
+	gst_object_unref (child);
+}
